@@ -30,7 +30,6 @@ from .components.scheduler import (
     individual_profile_doc,
     individual_profile_server_doc,
     individual_nbytes_doc,
-    individual_memory_use_doc,
     individual_cpu_doc,
     individual_nprocessing_doc,
     individual_workers_doc,
@@ -403,7 +402,6 @@ applications = {
     "/individual-profile": individual_profile_doc,
     "/individual-profile-server": individual_profile_server_doc,
     "/individual-nbytes": individual_nbytes_doc,
-    "/individual-memory-use": individual_memory_use_doc,
     "/individual-cpu": individual_cpu_doc,
     "/individual-nprocessing": individual_nprocessing_doc,
     "/individual-workers": individual_workers_doc,
