@@ -16,7 +16,7 @@ def serialize_cudf_dataframe(x):
 
 
 @cuda_deserialize.register((cudf.DataFrame, cudf.Series, cudf.groupby.groupby._Groupby))
-def serialize_cudf_dataframe(header, frames):
+def deserialize_cudf_dataframe(header, frames):
     with log_errors():
         cudf_typ = pickle.loads(header["type"])
         cudf_obj = cudf_typ.deserialize(header, frames)
