@@ -1,7 +1,4 @@
-from __future__ import print_function, division, absolute_import
-
 import logging
-from tornado import gen
 from .client import futures_of, wait
 from .utils import sync, tokey
 from .utils_comm import pack_data
@@ -77,20 +74,19 @@ class ReplayExceptionClient(object):
     def scheduler(self):
         return self.client.scheduler
 
-    @gen.coroutine
-    def _get_futures_error(self, future):
+    async def _get_futures_error(self, future):
         # only get errors for futures that errored.
         futures = [f for f in futures_of(future) if f.status == "error"]
         if not futures:
             raise ValueError("No errored futures passed")
-        out = yield self.scheduler.cause_of_failure(keys=[f.key for f in futures])
+        out = await self.scheduler.cause_of_failure(keys=[f.key for f in futures])
         deps, task = out["deps"], out["task"]
         if isinstance(task, dict):
             function, args, kwargs = _deserialize(**task)
-            raise gen.Return((function, args, kwargs, deps))
+            return (function, args, kwargs, deps)
         else:
             function, args, kwargs = _deserialize(task=task)
-            raise gen.Return((function, args, kwargs, deps))
+            return (function, args, kwargs, deps)
 
     def get_futures_error(self, future):
         """
@@ -122,16 +118,15 @@ class ReplayExceptionClient(object):
         """
         return self.client.sync(self._get_futures_error, future)
 
-    @gen.coroutine
-    def _recreate_error_locally(self, future):
-        yield wait(future)
-        out = yield self._get_futures_error(future)
+    async def _recreate_error_locally(self, future):
+        await wait(future)
+        out = await self._get_futures_error(future)
         function, args, kwargs, deps = out
         futures = self.client._graph_to_futures({}, deps)
-        data = yield self.client._gather(futures)
+        data = await self.client._gather(futures)
         args = pack_data(args, data)
         kwargs = pack_data(kwargs, data)
-        raise gen.Return((function, args, kwargs))
+        return (function, args, kwargs)
 
     def recreate_error_locally(self, future):
         """
