@@ -1,4 +1,4 @@
-from contextlib import contextmanager
+import asyncio
 from datetime import timedelta
 import random
 
@@ -10,72 +10,67 @@ from distributed.batched import BatchedSend
 from distributed.core import listen, connect, CommClosedError
 from distributed.metrics import time
 from distributed.utils import All
-from distributed.utils_test import gen_test, captured_logger
+from distributed.utils_test import captured_logger
 from distributed.protocol import to_serialize
 
 
 class EchoServer(object):
     count = 0
 
-    @gen.coroutine
-    def handle_comm(self, comm):
+    async def handle_comm(self, comm):
         while True:
             try:
-                msg = yield comm.read()
+                msg = await comm.read()
                 self.count += 1
-                yield comm.write(msg)
+                await comm.write(msg)
             except CommClosedError as e:
                 return
 
-    def listen(self):
+    async def listen(self):
         listener = listen("", self.handle_comm)
-        listener.start()
+        await listener.start()
         self.address = listener.contact_address
         self.stop = listener.stop
 
+    async def __aenter__(self):
+        await self.listen()
+        return self
 
-@contextmanager
-def echo_server():
-    server = EchoServer()
-    server.listen()
-
-    try:
-        yield server
-    finally:
-        server.stop()
+    async def __aexit__(self, exc, typ, tb):
+        self.stop()
 
 
-@gen_test()
-def test_BatchedSend():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_BatchedSend():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval=10)
         assert str(len(b.buffer)) in str(b)
         assert str(len(b.buffer)) in repr(b)
         b.start(comm)
 
-        yield gen.sleep(0.020)
+        await asyncio.sleep(0.020)
 
         b.send("hello")
         b.send("hello")
         b.send("world")
-        yield gen.sleep(0.020)
+        await asyncio.sleep(0.020)
         b.send("HELLO")
         b.send("HELLO")
 
-        result = yield comm.read()
+        result = await comm.read()
         assert result == ("hello", "hello", "world")
-        result = yield comm.read()
+        result = await comm.read()
         assert result == ("HELLO", "HELLO")
 
         assert b.byte_count > 1
 
 
-@gen_test()
-def test_send_before_start():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_send_before_start():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval=10)
 
@@ -83,120 +78,117 @@ def test_send_before_start():
         b.send("world")
 
         b.start(comm)
-        result = yield comm.read()
+        result = await comm.read()
         assert result == ("hello", "world")
 
 
-@gen_test()
-def test_send_after_stream_start():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_send_after_stream_start():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval=10)
 
         b.start(comm)
         b.send("hello")
         b.send("world")
-        result = yield comm.read()
+        result = await comm.read()
         if len(result) < 2:
-            result += yield comm.read()
+            result += await comm.read()
         assert result == ("hello", "world")
 
 
-@gen_test()
-def test_send_before_close():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_send_before_close():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval=10)
         b.start(comm)
 
         cnt = int(e.count)
         b.send("hello")
-        yield b.close()  # close immediately after sending
+        await b.close()  # close immediately after sending
         assert not b.buffer
 
         start = time()
         while e.count != cnt + 1:
-            yield gen.sleep(0.01)
+            await asyncio.sleep(0.01)
             assert time() < start + 5
 
         with pytest.raises(CommClosedError):
             b.send("123")
 
 
-@gen_test()
-def test_close_closed():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_close_closed():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval=10)
         b.start(comm)
 
         b.send(123)
-        yield comm.close()  # external closing
+        await comm.close()  # external closing
 
-        yield b.close()
+        await b.close()
         assert "closed" in repr(b)
         assert "closed" in str(b)
 
 
-@gen_test()
-def test_close_not_started():
+@pytest.mark.asyncio
+async def test_close_not_started():
     b = BatchedSend(interval=10)
-    yield b.close()
+    await b.close()
 
 
-@gen_test()
-def test_close_twice():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_close_twice():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval=10)
         b.start(comm)
-        yield b.close()
-        yield b.close()
+        await b.close()
+        await b.close()
 
 
 @pytest.mark.slow
-@gen_test(timeout=50)
-def test_stress():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_stress():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
         L = []
 
-        @gen.coroutine
-        def send():
+        async def send():
             b = BatchedSend(interval=3)
             b.start(comm)
             for i in range(0, 10000, 2):
                 b.send(i)
                 b.send(i + 1)
-                yield gen.sleep(0.00001 * random.randint(1, 10))
+                await asyncio.sleep(0.00001 * random.randint(1, 10))
 
-        @gen.coroutine
-        def recv():
+        async def recv():
             while True:
-                result = yield gen.with_timeout(timedelta(seconds=1), comm.read())
+                result = await gen.with_timeout(timedelta(seconds=1), comm.read())
                 L.extend(result)
                 if result[-1] == 9999:
                     break
 
-        yield All([send(), recv()])
+        await All([send(), recv()])
 
         assert L == list(range(0, 10000, 1))
-        yield comm.close()
+        await comm.close()
 
 
-@gen.coroutine
-def run_traffic_jam(nsends, nbytes):
+async def run_traffic_jam(nsends, nbytes):
     # This test eats `nsends * nbytes` bytes in RAM
     np = pytest.importorskip("numpy")
     from distributed.protocol import to_serialize
 
     data = bytes(np.random.randint(0, 255, size=(nbytes,)).astype("u1").data)
-    with echo_server() as e:
-        comm = yield connect(e.address)
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval=0.01)
         b.start(comm)
@@ -205,7 +197,7 @@ def run_traffic_jam(nsends, nbytes):
         for i in range(nsends):
             b.send(assoc(msg, "i", i))
             if np.random.random() > 0.5:
-                yield gen.sleep(0.001)
+                await asyncio.sleep(0.001)
 
         results = []
         count = 0
@@ -213,7 +205,7 @@ def run_traffic_jam(nsends, nbytes):
             # If this times out then I think it's a backpressure issue
             # Somehow we're able to flood the socket so that the receiving end
             # loses some of our messages
-            L = yield gen.with_timeout(timedelta(seconds=5), comm.read())
+            L = await gen.with_timeout(timedelta(seconds=5), comm.read())
             count += 1
             results.extend(r["i"] for r in L)
 
@@ -222,45 +214,45 @@ def run_traffic_jam(nsends, nbytes):
 
         assert results == list(range(nsends))
 
-        yield comm.close()  # external closing
-        yield b.close()
+        await comm.close()  # external closing
+        await b.close()
 
 
-@gen_test()
-def test_sending_traffic_jam():
-    yield run_traffic_jam(50, 300000)
+@pytest.mark.asyncio
+async def test_sending_traffic_jam():
+    await run_traffic_jam(50, 300000)
 
 
 @pytest.mark.slow
-@gen_test()
-def test_large_traffic_jam():
-    yield run_traffic_jam(500, 1500000)
+@pytest.mark.asyncio
+async def test_large_traffic_jam():
+    await run_traffic_jam(500, 1500000)
 
 
-@gen_test()
-def test_serializers():
-    with echo_server() as e:
-        comm = yield connect(e.address)
+@pytest.mark.asyncio
+async def test_serializers():
+    async with EchoServer() as e:
+        comm = await connect(e.address)
 
         b = BatchedSend(interval="10ms", serializers=["msgpack"])
         b.start(comm)
 
         b.send({"x": to_serialize(123)})
         b.send({"x": to_serialize("hello")})
-        yield gen.sleep(0.100)
+        await asyncio.sleep(0.100)
 
         b.send({"x": to_serialize(lambda x: x + 1)})
 
         with captured_logger("distributed.protocol") as sio:
-            yield gen.sleep(0.100)
+            await asyncio.sleep(0.100)
 
         value = sio.getvalue()
         assert "serialize" in value
         assert "type" in value
         assert "function" in value
 
-        msg = yield comm.read()
+        msg = await comm.read()
         assert list(msg) == [{"x": 123}, {"x": "hello"}]
 
         with pytest.raises(gen.TimeoutError):
-            msg = yield gen.with_timeout(timedelta(milliseconds=100), comm.read())
+            msg = await gen.with_timeout(timedelta(milliseconds=100), comm.read())
